@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010  Roman Zimbelmann <romanz@lavabit.com>
+# Copyright (C) 2009, 2010, 2011  Roman Zimbelmann <romanz@lavabit.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -32,7 +32,10 @@ class Snow(ColorScheme):
 			if context.directory:
 				attr |= bold
 
-		if context.highlight:
+		elif context.highlight:
+			attr |= reverse
+
+		elif context.in_titlebar and context.tab and context.good:
 			attr |= reverse
 
 		return fg, bg, attr
