@@ -1,4 +1,4 @@
-# Copyright (C) 2009, 2010  Roman Zimbelmann <romanz@lavabit.com>
+# Copyright (C) 2009, 2010, 2011  Roman Zimbelmann <romanz@lavabit.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,19 +14,17 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import os.path
-import stat
-from stat import S_ISLNK, S_ISDIR
 from os import stat as os_stat, lstat as os_lstat
-from os.path import join, isdir, basename
 from collections import deque
 from time import time
 
+from ranger.fsobject import BAD_INFO
 from ranger.core.loader import Loadable
 from ranger.ext.mount_path import mount_path
-from ranger.fsobject import BAD_INFO, File, FileSystemObject
+from ranger.fsobject import File, FileSystemObject
 from ranger.core.shared import SettingsAware
 from ranger.ext.accumulator import Accumulator
-import ranger.fsobject
+from ranger.ext.lazy_property import lazy_property
 
 def sort_by_basename(path):
 	"""returns path.basename (for sorting)"""
@@ -46,7 +44,7 @@ def sort_naturally(path):
 def sort_naturally_icase(path):
 	return path.basename_natural_lower
 
-def accept_file(fname, hidden_filter, name_filter):
+def accept_file(fname, dirname, hidden_filter, name_filter):
 	if hidden_filter:
 		try:
 			if hidden_filter.search(fname):
@@ -86,6 +84,8 @@ class Directory(FileSystemObject, Accumulator, Loadable, SettingsAware):
 		'natural': sort_naturally,
 		'size': lambda path: -path.size,
 		'mtime': lambda path: -(path.stat and path.stat.st_mtime or 1),
+		'ctime': lambda path: -(path.stat and path.stat.st_ctime or 1),
+		'atime': lambda path: -(path.stat and path.stat.st_atime or 1),
 		'type': lambda path: path.mimetype or '',
 	}
 
@@ -143,6 +143,8 @@ class Directory(FileSystemObject, Accumulator, Loadable, SettingsAware):
 			del self.marked_items[:]
 			self._clear_marked_items()
 
+	# XXX: Is it really necessary to have the marked items in a list?
+	# Can't we just recalculate them with [f for f in self.files if f.marked]?
 	def _gc_marked_items(self):
 		for item in list(self.marked_items):
 			if item.path not in self.filenames:
@@ -163,6 +165,7 @@ class Directory(FileSystemObject, Accumulator, Loadable, SettingsAware):
 		else:
 			return []
 
+	# XXX: Check for possible race conditions
 	def load_bit_by_bit(self):
 		"""
 		Returns a generator which load a part of the directory
@@ -181,9 +184,12 @@ class Directory(FileSystemObject, Accumulator, Loadable, SettingsAware):
 
 				hidden_filter = not self.settings.show_hidden \
 						and self.settings.hidden_filter
+				filelist = os.listdir(mypath)
+				self.size = len(filelist)
+				self.infostring = ' %d' % self.size
 				filenames = [mypath + (mypath == '/' and fname or '/' + fname)\
-						for fname in os.listdir(mypath) \
-						if accept_file(fname, hidden_filter, self.filter)]
+						for fname in filelist if accept_file(
+							fname, mypath, hidden_filter, self.filter)]
 				yield
 
 				self.load_content_mtime = os.stat(mypath).st_mtime
@@ -249,6 +255,7 @@ class Directory(FileSystemObject, Accumulator, Loadable, SettingsAware):
 
 		finally:
 			self.loading = False
+			self.fm.signal_emit("finished_loading_dir", directory=self)
 
 	def unload(self):
 		self.loading = False
@@ -320,6 +327,33 @@ class Directory(FileSystemObject, Accumulator, Loadable, SettingsAware):
 		else:
 			self.correct_pointer()
 
+	@lazy_property
+	def size(self):
+		try:
+			size = len(os.listdir(self.path))  # bite me
+		except OSError:
+			self.infostring = BAD_INFO
+			self.accessible = False
+			self.runnable = False
+			return 0
+		else:
+			self.infostring = ' %d' % size
+			self.accessible = True
+			self.runnable = True
+			return size
+
+	@lazy_property
+	def infostring(self):
+		self.size  # trigger the lazy property initializer
+		if self.is_link:
+			return '->' + self.infostring
+		return self.infostring
+
+	@lazy_property
+	def runnable(self):
+		self.size  # trigger the lazy property initializer
+		return self.runnable
+
 	def sort_if_outdated(self):
 		"""Sort the containing files if they are outdated"""
 		if self.order_outdated:
