@@ -1,4 +1,5 @@
-# Copyright (C) 2009, 2010  Roman Zimbelmann <romanz@lavabit.com>
+# -*- encoding: utf8 -*-
+# Copyright (C) 2009, 2010, 2011  Roman Zimbelmann <romanz@lavabit.com>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,15 +21,15 @@ from time import time
 from . import Widget
 from .pager import Pager
 from ranger.fsobject import BAD_INFO
+from ranger.ext.widestring import WideString
 
 class BrowserColumn(Pager):
 	main_column = False
 	display_infostring = False
 	scroll_begin = 0
 	target = None
-	tagged_marker = '*'
 	last_redraw_time = -1
-	ellipsis = "~"
+	ellipsis = { False: '~', True: '…' }
 
 	old_dir = None
 	old_cf = None
@@ -159,17 +160,12 @@ class BrowserColumn(Pager):
 			Pager.close(self)
 			return
 
-		try:
-			f = self.target.get_preview_source(self.wid, self.hei)
-		except:
-			raise # XXX
+		f = self.target.get_preview_source(self.wid, self.hei)
+		if f is None:
 			Pager.close(self)
 		else:
-			if f is None:
-				Pager.close(self)
-			else:
-				self.set_source(f)
-				Pager.draw(self)
+			self.set_source(f)
+			Pager.draw(self)
 
 	def _draw_directory(self):
 		"""Draw the contents of a directory"""
@@ -205,6 +201,7 @@ class BrowserColumn(Pager):
 		self._set_scroll_begin()
 
 		copied = [f.path for f in self.env.copy]
+		ellipsis = self.ellipsis[self.settings.unicode_ellipsis]
 
 		selected_i = self.target.pointer
 		for line in range(self.hei):
@@ -226,25 +223,30 @@ class BrowserColumn(Pager):
 			text = drawn.basename
 			tagged = self.fm.tags and drawn.realpath in self.fm.tags
 
+			if tagged:
+				tagged_marker = self.fm.tags.marker(drawn.realpath)
+
 			space = self.wid - len(infostring)
 			if self.main_column:
 				space -= 2
+			elif self.settings.display_tags_in_all_columns:
+				space -= 1
 
-			if len(text) > space:
-				text = text[:space-1] + self.ellipsis
+#			if len(text) > space:
+#				text = text[:space-1] + self.ellipsis
 
 			if i == selected_i:
 				this_color.append('selected')
 
 			if drawn.marked:
 				this_color.append('marked')
-				if self.main_column:
+				if self.main_column or self.settings.display_tags_in_all_columns:
 					text = " " + text
 
 			if tagged:
 				this_color.append('tagged')
-				if self.main_column:
-					text = self.tagged_marker + text
+				if self.main_column or self.settings.display_tags_in_all_columns:
+					text = tagged_marker + text
 
 			if drawn.is_directory:
 				this_color.append('directory')
@@ -269,14 +271,16 @@ class BrowserColumn(Pager):
 				this_color.append('link')
 				this_color.append(drawn.exists and 'good' or 'bad')
 
-			string = drawn.basename
-			if self.main_column:
+			wtext = WideString(text)
+			if len(wtext) > space:
+				wtext = wtext[:space - 1] + ellipsis
+			if self.main_column or self.settings.display_tags_in_all_columns:
 				if tagged:
-					self.addnstr(line, 0, text, self.wid - 2)
+					self.addstr(line, 0, str(wtext))
 				elif self.wid > 1:
-					self.addnstr(line, 1, text, self.wid - 2)
+					self.addstr(line, 1, str(wtext))
 			else:
-				self.addnstr(line, 0, text, self.wid)
+				self.addstr(line, 0, str(wtext))
 
 			if infostring:
 				x = self.wid - 1 - len(infostring)
@@ -290,9 +294,10 @@ class BrowserColumn(Pager):
 				start, wid = bad_info_color
 				self.color_at(line, start, wid, this_color, 'badinfo')
 
-			if self.main_column and tagged and self.wid > 2:
+			if (self.main_column or self.settings.display_tags_in_all_columns) \
+					and tagged and self.wid > 2:
 				this_color.append('tag_marker')
-				self.color_at(line, 0, len(self.tagged_marker), this_color)
+				self.color_at(line, 0, len(tagged_marker), this_color)
 
 			self.color_reset()
 
